#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jun 10 16:50:48 2024

@author: anton
"""

from SLiCAP import *

from gain_zin_FdB import solutions, Rs

fmin   = 1e6
fmax   = 250e6
F_dB   = 1.5 # Noise figure including CS stage noise
L      = 0.22e-6
VGSmax = 1

fileName = "NMOS_input"

i1 = instruction()
i1.setCircuit(fileName + ".cir")
i1.setSimType("symbolic")
i1.setGainType("vi")
i1.setDataType("noise")
i1.setSource("V1")
i1.setDetector("Io_N1")
fT   = i1.getParValue("f_T_XU1")
gm   = i1.getParValue("g_m_XU1")
IC   = i1.getParValue("IC_XU1")
ciss = i1.getParValue("c_iss_XU1")
i1.defPar("R_s", Rs)
i1.defPars(solutions)

# Design the transistor
IC_init  = 1
IC_found = 0.5
error = False
i = 0
while sp.Abs(1-IC_found/IC_init) > 0.01 and not error:
    IC_init = IC_found
    i1.delPar("g_m_XU1")
    i1.delPar("c_iss_XU1")
    i1.defPar("IC_XU1", IC_init)
    i1.setSimType("numeric")
    noiseResult = i1.execute()
    F = (rmsNoise(noiseResult, "inoise", fmin, fmax)/rmsNoise(noiseResult, "inoise", fmin, fmax, source="I_noise_R1"))**2
    gm_ciss = sp.solve(F-10**(F_dB/10), sp.Symbol("g_m_XU1"))[0]
    num, den = gm_ciss.as_numer_denom()
    c_iss_min = float(sp.solve(den, sp.Symbol("c_iss_XU1"))[0])
    costs = gm_ciss*sp.Symbol("c_iss_XU1")
    c_min_costs_func = sp.simplify(sp.diff(costs, sp.Symbol("c_iss_XU1")))
    c_min_costs_func = sp.lambdify(sp.Symbol("c_iss_XU1"), c_min_costs_func)
    c_min_costs = fsolve(c_min_costs_func, c_iss_min*1.1)[0]
    i1.defPar("c_iss_XU1", ciss)
    i1.defPar("L", L)
    W = sp.solve(i1.getParValue("c_iss_XU1")-c_min_costs, sp.Symbol("W"))[0]
    i1.defPar("W", W)
    i1.defPar("g_m_XU1", gm)
    ID = sp.solve(i1.getParValue("g_m_XU1") - gm_ciss.subs(sp.Symbol("c_iss_XU1"), c_min_costs))[0]
    i1.defPar("ID", ID)
    i1.defPar("IC_XU1", IC)
    IC_found = i1.getParValue("IC_XU1")
    # Calculate VGS
    VGS = i1.getParValue("V_GS_XU1")
    g_m = i1.getParValue("g_m_XU1")
    f_T = i1.getParValue("f_T_XU1")
    # Check VGS limit
    if sp.Abs(VGS) > VGSmax:
        print("\nVGS requirement cannot be met.")
        error = True
    # Initialize for next loop
    i1.defPar("IC_XU1", IC)
    i1.defPar("c_iss_XU1", ciss)
    i1.defPar("g_m_XU1", gm)
    i1.delPar("W")
    i1.delPar("ID")
    i1.defPar("f_T_XU1", fT)
    i += 1
    print("IC:", sp.N(IC_found,3))

if  not error:
    print("\nFound results after %s iterations:"%(str(i)))
    print("Channel width           :", sp.N(W,3))
    print("Channel length          :", sp.N(L,3))
    print("Drain current           :", sp.N(ID,3))
    print("g_m                     :", sp.N(g_m,3))
    print("c_iss                   :", sp.N(c_min_costs,3))
    print("g_m/ID                  :", sp.N(g_m/ID ,3))
    print("Inversion coefficient   :", sp.N(IC_found,3))
    print("VGS                     :", sp.N(VGS,3))
    print("f_T                     :", sp.N(f_T,3))
