# -*- coding: utf-8 -*-
"""
Created on Mon May 27 13:11:51 2024

@author: Paul
"""

from SLiCAP import *

# Requirements:
Rs  = 150  # Half the source impedance of the balanced version
Zin = 150  # Half the input impedance of the balanced version
Gm  = 0.08 # Twice the transconductance of tha balanced version
FdB = 0.5  # Maximum contribution of feedback network to the noise figure, 
           # must be larger than '0' and smaller than 2.5 dB

filename='gain_zin_FdB'

i1=instruction()
i1.setCircuit(filename +'.cir')
i1.setSimType('numeric')
i1.defPar("R_s", Rs)
# Finding input impedance equation
i1.setGainType('gain') # gain = transfer from source to detector
i1.setDataType('laplace')
i1.setSource('V1')

# Determination Z_in
i1.setDetector('I_V1') # Current through V1 = negative input current amplifier
I_s = i1.execute()
i1.setDetector("V_i")  # Input voltage amplifier
V_in = i1.execute()
Z_in = sp.simplify(-V_in.laplace/I_s.laplace)
print('\nZ_i = Vi / (-I_V1) = ', Z_in)

# Determination transadmittance
i1.setDetector('Io_N1') # Norator current = negative output current amplifier
G_m = sp.simplify(-i1.execute().laplace)
print('\nGm = ', G_m)

# Determination of the noise figure (not in dB)
i1.setGainType('vi')
i1.setDataType('noise')
noiseresult=i1.execute()
F = sp.simplify(noiseresult.inoise/noiseresult.inoiseTerms['I_noise_R1'])
print('\nFnoise = ', F)

# Solve Z_in = Zin and G_m = Gm
solutions = sp.solve([Z_in - Zin, G_m - Gm])

# Find circuit conditions for the noise figure
if len(solutions) > 0:
    
    # Substitute solutions in exprerssion for noise figure
    solutions = solutions[0]
    F = F.subs(solutions) 
    
    # Solve F <= 10**(FdB/10)
    solution = sp.reduce_inequalities([F <= 10**(FdB/10)])
    variable = list(solution.atoms(sp.Symbol))[0]
    solutions[variable] = sp.reduce_inequalities([F <= 10**(FdB/10)])
    
    # Print the results
    print("\nFound solution:")
    for param in solutions.keys():
        print(param, '=', solutions[param])
        
    # Select the free circuit variable and check if the selected value is 
    # positive and satisfies the condition for the noise figure
    print("\nYou can now select a value for " + str(variable) + " and calculate the required value for Ra from the above expression.")
    selected_value = -1
    while selected_value <= 0 or not solutions[variable].subs(variable, selected_value):
        selected_value = float(input("Enter a value for " + str(variable) + "\n>>> "))
    solutions[variable] = sp.N(selected_value)
    
    # Calculate the achieved noise figure and print the results
    solutions[sp.Symbol('FdB')] = sp.N(10*sp.log(F)/sp.log(10))
    for param in solutions.keys():
        solutions[param] = sp.N(fullSubs(solutions[param], solutions))
        print(param, '=', sp.N(solutions[param], 3))